@@ -21,7 +21,9 @@
  * Author: Matthias Clasen
  */
 
+#ifndef GLIB_DISABLE_DEPRECATION_WARNINGS
 #define GLIB_DISABLE_DEPRECATION_WARNINGS
+#endif
 
 #include "glib.h"
 #include "glib-private.h"
@@ -159,7 +161,7 @@ test_appname (void)
 static void
 test_tmpdir (void)
 {
-  g_test_bug ("627969");
+  g_test_bug ("https://bugzilla.gnome.org/show_bug.cgi?id=627969");
   g_assert_cmpstr (g_get_tmp_dir (), !=, "");
 }
 
@@ -244,6 +246,11 @@ test_find_program (void)
   gchar *res;
 
 #ifdef G_OS_UNIX
+  gchar *relative_path;
+  gchar *absolute_path;
+  gchar *cwd;
+  gsize i;
+
   res = g_find_program_in_path ("sh");
   g_assert (res != NULL);
   g_free (res);
@@ -251,6 +258,27 @@ test_find_program (void)
   res = g_find_program_in_path ("/bin/sh");
   g_assert (res != NULL);
   g_free (res);
+
+  cwd = g_get_current_dir ();
+  absolute_path = g_find_program_in_path ("sh");
+  relative_path = g_strdup (absolute_path);
+  for (i = 0; cwd[i] != '\0'; i++)
+    {
+      if (cwd[i] == '/' && cwd[i + 1] != '\0')
+        {
+          gchar *relative_path_2 = g_strconcat ("../", relative_path, NULL);
+          g_free (relative_path);
+          relative_path = relative_path_2;
+        }
+    }
+  res = g_find_program_in_path (relative_path);
+  g_assert_nonnull (res);
+  g_assert_true (g_path_is_absolute (res));
+  g_free (cwd);
+  g_free (absolute_path);
+  g_free (relative_path);
+  g_free (res);
+
 #else
   /* There's not a lot we can search for that would reliably work both
    * on real Windows and mingw.
@@ -827,7 +855,6 @@ main (int   argc,
   g_set_prgname (argv[0]);
 
   g_test_init (&argc, &argv, NULL);
-  g_test_bug_base ("http://bugzilla.gnome.org/");
 
   g_test_add_func ("/utils/language-names", test_language_names);
   g_test_add_func ("/utils/locale-variants", test_locale_variants);
